#include "DatatypeConverter.h"

namespace armarx::aron::converter
{
    void DatatypeConverter::visitAronVariant(const type::ObjectPtr& el)
    {
        auto dict = aron::make_dict(el->getPath());
        auto type = aron::make_string(el->getFullName(), Path(el->getPath(), {typeMemberName}));
        dict->addElement(typeMemberName, type);

        for (const auto& [key, child] : el->getMemberTypes())
        {
            DatatypeConverter converter;
            aron::type::visit(converter, child);

            dict->addElement(key, converter.latest);
        }

        latest = dict;
    }

    void DatatypeConverter::visitAronVariant(const type::DictPtr& el)
    {
        auto dict = aron::make_dict(el->getPath());
        auto type = aron::make_string(el->getFullName(), Path(el->getPath(), {typeMemberName}));
        dict->addElement(typeMemberName, type);

        auto acceptedType = el->getAcceptedType();
        DatatypeConverter converter;
        aron::type::visit(converter, acceptedType);
        dict->addElement(acceptedTypeMemberName, converter.latest);

        latest = dict;
    }

    void DatatypeConverter::visitAronVariant(const type::ListPtr& el)
    {
        auto dict = aron::make_dict(el->getPath());
        auto type = aron::make_string(el->getFullName(), Path(el->getPath(), {typeMemberName}));
        dict->addElement(typeMemberName, type);

        auto acceptedType = el->getAcceptedType();
        DatatypeConverter converter;
        aron::type::visit(converter, acceptedType);
        dict->addElement(acceptedTypeMemberName, converter.latest);

        latest = dict;
    }

    void DatatypeConverter::visitAronVariant(const type::PairPtr& el)
    {
        auto dict = aron::make_dict(el->getPath());
        auto type = aron::make_string(el->getFullName(), Path(el->getPath(), {typeMemberName}));
        dict->addElement(typeMemberName, type);

        auto acceptedType1 = el->getFirstAcceptedType();
        DatatypeConverter converter1;
        aron::type::visit(converter1, acceptedType1);
        dict->addElement(firstAcceptedTypeMemberName, converter1.latest);

        auto acceptedType2 = el->getSecondAcceptedType();
        DatatypeConverter converter2;
        aron::type::visit(converter2, acceptedType2);
        dict->addElement(secondAcceptedTypeMemberName, converter2.latest);

        latest = dict;
    }

    void DatatypeConverter::visitAronVariant(const type::TuplePtr& el)
    {
        auto dict = aron::make_dict(el->getPath());
        auto type = aron::make_string(el->getFullName(), Path(el->getPath(), {typeMemberName}));
        dict->addElement(typeMemberName, type);

        int i = 0;
        for (const auto& child : el->getAcceptedTypes())
        {
            DatatypeConverter converter;
            aron::type::visit(converter, child);

            dict->addElement(acceptedTypeMemberName + "_" + std::to_string(i++), converter.latest);
        }

        latest = dict;
    }

    void DatatypeConverter::visitAronVariant(const type::MatrixPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void DatatypeConverter::visitAronVariant(const type::NDArrayPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void DatatypeConverter::visitAronVariant(const type::QuaternionPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void DatatypeConverter::visitAronVariant(const type::PointCloudPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void DatatypeConverter::visitAronVariant(const type::ImagePtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void DatatypeConverter::visitAronVariant(const type::IntEnumPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void DatatypeConverter::visitAronVariant(const type::IntPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void DatatypeConverter::visitAronVariant(const type::LongPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void DatatypeConverter::visitAronVariant(const type::FloatPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void DatatypeConverter::visitAronVariant(const type::DoublePtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void DatatypeConverter::visitAronVariant(const type::BoolPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void DatatypeConverter::visitAronVariant(const type::StringPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }

    void DatatypeConverter::visitAronVariant(const type::AnyObjectPtr& el)
    {
        latest = aron::make_string(el->getFullName(), el->getPath());
    }
}
