/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller ( fabian dot peller at kit dot edu )
 * @date       2023
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <algorithm>
#include <vector>

#include <RobotAPI/libraries/aron/core/data/variant/All.h>
#include <RobotAPI/libraries/aron/core/type/visitor/variant/VariantVisitor.h>

namespace armarx::aron::converter
{
    class DatatypeConverter :
        public aron::type::ConstVariantVisitor
    {
        const std::string typeMemberName = "_aron_type";
        const std::string acceptedTypeMemberName = "_aron_accepted_type";
        const std::string firstAcceptedTypeMemberName = "_aron_first_accepted_type";
        const std::string secondAcceptedTypeMemberName = "_aron_second_accepted_type";

        void visitAronVariant(const type::ObjectPtr&) override;
        void visitAronVariant(const type::DictPtr&) override;
        void visitAronVariant(const type::ListPtr&) override;
        void visitAronVariant(const type::PairPtr&) override;
        void visitAronVariant(const type::TuplePtr&) override;
        void visitAronVariant(const type::MatrixPtr&) override;
        void visitAronVariant(const type::NDArrayPtr&) override;
        void visitAronVariant(const type::QuaternionPtr&) override;
        void visitAronVariant(const type::PointCloudPtr&) override;
        void visitAronVariant(const type::ImagePtr&) override;
        void visitAronVariant(const type::IntEnumPtr&) override;
        void visitAronVariant(const type::IntPtr&) override;
        void visitAronVariant(const type::LongPtr&) override;
        void visitAronVariant(const type::FloatPtr&) override;
        void visitAronVariant(const type::DoublePtr&) override;
        void visitAronVariant(const type::BoolPtr&) override;
        void visitAronVariant(const type::StringPtr&) override;
        void visitAronVariant(const type::AnyObjectPtr&) override;

    public:
        aron::data::VariantPtr latest;
    };
}
